
#include "particle.h"

#include "input.h"

bool LoadParticleSystem(char *File, PARTICLE_SYSTEM_DESC *psd)
{
	
	ifstream DescFile(File, ios::nocreate);
	if(DescFile.fail())
		return 0;

	DescFile >> psd->Name;
	DescFile >> psd->MaxParticles;
	DescFile >> psd->EmissionRate;
	DescFile >> psd->Mode;
	DescFile >> psd->ColorStart.x;
	DescFile >> psd->ColorStart.y;
	DescFile >> psd->ColorStart.z;
	DescFile >> psd->ColorEnd.x;
	DescFile >> psd->ColorEnd.y;
	DescFile >> psd->ColorEnd.z;
	DescFile >> psd->AlphaStart;	 
	DescFile >> psd->AlphaEnd;
	DescFile >> psd->Environment.x;
	DescFile >> psd->Environment.y;
	DescFile >> psd->EmitterAngle;
	DescFile >> psd->SpeedBase;
	DescFile >> psd->SpeedVary;
	DescFile >> psd->LifeBase;
	DescFile >> psd->LifeVary;
	DescFile >> psd->SizeStart;      
	DescFile >> psd->SizeEnd;
	DescFile >> psd->TextureFileName;
	DescFile >> psd->MaskFileName;

	DescFile.close();

	psd->Texture.Load(psd->TextureFileName);
	psd->Mask.Load(psd->MaskFileName);

	return true;
}

CParticleSystem::CParticleSystem(PARTICLE_SYSTEM_DESC *psd)
{
	Load(psd);
}

void CParticleSystem::Load(PARTICLE_SYSTEM_DESC *psd)
{
	Emitting = false;
	NumParticles = 0;
	MaxParticles = psd->MaxParticles;
	EmissionRate = psd->EmissionRate;
	EmitterAngle = psd->EmitterAngle;
	Mode = psd->Mode;
	Texture = &(psd->Texture);
	Mask = &(psd->Mask);
	ColorStart = psd->ColorStart;
	ColorEnd = psd->ColorEnd;
	AlphaStart = psd->AlphaStart;
	AlphaEnd = psd->AlphaEnd;
	SizeStart = psd->SizeStart;
	SizeEnd = psd->SizeEnd;
	LifeBase = psd->LifeBase;
	LifeVary = psd->LifeVary;
	SpeedBase = psd->SpeedBase;
	SpeedVary = psd->SpeedVary;
	Environment = psd->Environment;
	
	Emitter = CVector(0,0,0);

	ParticleList = new CParticle[MaxParticles];
}

CParticleSystem::~CParticleSystem()
{
	Destroy();
}

void CParticleSystem::Destroy()
{
	if(ParticleList)
		delete [] ParticleList;
	ParticleList = 0;
}

void CParticleSystem::Position(CVector NewPos)
{
	Emitter = NewPos;
}

void CParticleSystem::Start()
{
	Emitting = true;
}

void CParticleSystem::Stop()
{
	Emitting = false;
}

void CParticleSystem::Update(float time)
{
	static float MaxToEmit = 0;

	MaxToEmit += EmissionRate*time;

	for(int i = MaxParticles-1; i >= 0; i--)
	{
		CParticle &Temp = ParticleList[i]; // to avoid constant dereferencing

		if(Temp.Alive == true)
		{
			Temp.Location += Temp.Velocity*time;
			Temp.Velocity += Environment*time;
			Temp.Color += (ColorEnd-ColorStart)/Temp.Lifespan*time;
			Temp.Alpha += (AlphaEnd-AlphaStart)/Temp.Lifespan*time;
			Temp.Size += (SizeEnd-SizeStart)/Temp.Lifespan*time;
			Temp.Age += time;
			if(Temp.Age >= Temp.Lifespan)
			{
				Temp.Alive = false;
				NumParticles--;
			}
		}
		else if(Emitting)
		{
			if(MaxToEmit > 0)
			{				
				MaxToEmit--;
				Temp.Alive = true;
				NumParticles++;
				Temp.Age = 0;
				Temp.Size = SizeStart;
				Temp.Color = ColorStart;
				Temp.Alpha = AlphaStart;
				Temp.Lifespan = LifeBase + LifeVary*randfloat();

				Temp.Location = Emitter;
				
				float Yaw = PI*randfloat()*2.0f;
				float Pitch = DEGTORAD*((EmitterAngle*randfloat())+90.0f);

				Temp.Velocity.x = SpeedBase * cosf(Pitch);
				Temp.Velocity.y = sinf(Pitch) * cosf(Yaw) * SpeedBase;
				Temp.Velocity.z = sinf(Pitch) * sinf(Yaw) * SpeedBase;
	
				float Speed = SpeedBase + randfloat()*SpeedVary;

				Temp.Velocity *= Speed;
			}
		}
	}

	if(NumParticles >= MaxParticles)
		Emitting = false;
	if(!Emitting && Mode != EMIT_EXPLOSION && NumParticles < MaxParticles)
		Emitting = true;
}

void CParticleSystem::Draw()
{
	float Size = 0.0f;
	CVector UL, LL, LR, UR;
	int i;

	float ModelMatrix[16];
	glGetFloatv(GL_MODELVIEW_MATRIX, ModelMatrix);
	CVector Right(ModelMatrix[0], ModelMatrix[4], ModelMatrix[8]);
	CVector Up(ModelMatrix[1], ModelMatrix[5], ModelMatrix[9]);

	glPushAttrib(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
	glDepthMask(GL_FALSE);

	Texture->Use();

    glBlendFunc(GL_SRC_ALPHA, GL_DST_ALPHA);
	glEnable(GL_BLEND);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	glBegin(GL_QUADS);
		for(i = MaxParticles-1; i >= 0; i--)
		{
			CParticle &Temp = ParticleList[i]; // to avoid constant dereferencing
			if(Temp.Alive)
			{
				Size = Temp.Size;
				glColor4f(Temp.Color.x, 
					      Temp.Color.y, 
						  Temp.Color.z, 
						  Temp.Alpha);
				glTexCoord2f(0,1);
				UL = Temp.Location - (Right - Up) * Size;
				glVertex3f(UL.x, UL.y, UL.z);
				glTexCoord2f(0,0);
				LL = Temp.Location - (Right + Up) * Size;
				glVertex3f(LL.x, LL.y, LL.z);
				glTexCoord2f(1,0);
				LR = Temp.Location + (Right - Up) * Size;
				glVertex3f(LR.x, LR.y, LR.z);
				glTexCoord2f(1,1);
				UR = Temp.Location + (Right + Up) * Size;
				glVertex3f(UR.x, UR.y, UR.z);
			}
		}
	glEnd();

	glPopAttrib();
}



	